@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header$
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.45 2010/04/13 04:14:08 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -74,7 +74,7 @@ static void connection_count_down(void);
 /*
  * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
-static int exit_request;
+volatile sig_atomic_t exit_request = 0;
 
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
@@ -169,17 +169,13 @@ void do_child(int unix_fd, int inet_fd)
 	for (;;)
 	{
 		int connection_reuse = 1;
-		int ssl_request = 0;
 		StartupPacket *sp;
 
-		/* pgpool stop request already sent? */
-		if (exit_request)
-		{
-			die(0);
-			child_exit(0);
-		}
-
 		idle = 1;
+
+		/* pgpool stop request already sent? */
+		check_stop_request();
+
 		accepted = 0;
 
 		/* perform accept() */
@@ -238,25 +234,10 @@ void do_child(int unix_fd, int inet_fd)
 		}
 
 		/* SSL? */
-		if (sp->major == 1234 && sp->minor == 5679)
+		if (sp->major == 1234 && sp->minor == 5679 && !frontend->ssl_active)
 		{
-			/* SSL not supported */
-			pool_debug("SSLRequest: sent N; retry startup");
-			if (ssl_request)
-			{
-				pool_close(frontend);
-				pool_free_startup_packet(sp);
-				connection_count_down();
-				continue;
-			}
-
-			/*
-			 * say to the frontend "we do not suppport SSL"
-			 * note that this is not a NOTICE response despite it's an 'N'!
-			 */
-			pool_write_and_flush(frontend, "N", 1);
-			ssl_request = 1;
-			pool_free_startup_packet(sp);
+			pool_debug("SSLRequest from client");
+			pool_ssl_negotiate_serverclient(frontend);
 			goto retry_startup;
 		}
 
@@ -438,7 +419,7 @@ void do_child(int unix_fd, int inet_fd)
 						!strcmp(sp->database, "postgres") ||
 						!strcmp(sp->database, "regression"))
 					{
-						reset_variables();
+						reset_connection();
 						pool_close(frontend);
 						pool_send_frontend_exits(backend);
 						pool_discard_cp(sp->user, sp->database, sp->major);
@@ -1110,9 +1091,9 @@ void cancel_request(CancelPacket *sp)
 			continue;
 
 		if (*(BACKEND_INFO(i).backend_hostname) == '\0')
-			fd = connect_unix_domain_socket(i);
+			fd = connect_unix_domain_socket(i, TRUE);
 		else
-			fd = connect_inet_domain_socket(i);
+			fd = connect_inet_domain_socket(i, TRUE);
 
 		if (fd < 0)
 		{
@@ -1171,6 +1152,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 			/* mark this is a backend connection */
 			CONNECTION(backend, i)->isbackend = 1;
+			pool_ssl_negotiate_clientserver(CONNECTION(backend, i));
 
 			/*
 			 * save startup packet info
@@ -1202,13 +1184,13 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 }
 
 /*
- * signal handler for SIGINT and SIGQUUT
+ * signal handler for SIGTERM, SIGINT and SIGQUUT
  */
 static RETSIGTYPE die(int sig)
 {
-	exit_request = 1;
+	pool_debug("child received shutdown request signal %d", sig);
 
-	pool_debug("child receives shutdown request signal %d", sig);
+	exit_request = sig;
 
 	switch (sig)
 	{
@@ -1216,7 +1198,6 @@ static RETSIGTYPE die(int sig)
 			if (idle == 0)
 			{
 				pool_debug("child receives smart shutdown request but it's not in idle state");
-				return;
 			}
 			break;
 
@@ -1225,14 +1206,9 @@ static RETSIGTYPE die(int sig)
 			child_exit(0);
 			break;
 		default:
+			pool_error("die() received unknown signal: %d", sig);
 			break;
 	}
-
-	/*
-	 * child_exit() does this. So we don't need it.
-	 * send_frontend_exits();
-	 */
-	child_exit(0);
 }
 
 /*
@@ -1429,11 +1405,11 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	 */
 	if (*hostname == '\0')
 	{
-		fd = connect_unix_domain_socket_by_port(port, pool_config->backend_socket_dir);
+		fd = connect_unix_domain_socket_by_port(port, pool_config->backend_socket_dir, TRUE);
 	}
 	else
 	{
-		fd = connect_inet_domain_socket_by_port(hostname, port);
+		fd = connect_inet_domain_socket_by_port(hostname, port, TRUE);
 	}
 
 	if (fd < 0)
@@ -1445,6 +1421,7 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	cp->con = pool_open(fd);
 	cp->closetime = 0;
 	cp->con->isbackend = 1;
+	pool_ssl_negotiate_clientserver(cp->con);
 
 	/*
 	 * build V3 startup packet
@@ -1853,3 +1830,22 @@ static RETSIGTYPE reload_config_handler(int sig)
 {
 	got_sighup = 1;
 }
+
+/*
+ * Exit myself if SIGTERM, SIGINT or SIGQUIT has been sent
+ */
+void check_stop_request(void)
+{
+    /*
+	 * If smart shutdown was requested but we are not in idle state,
+	 * do not exit
+	 */
+	if (exit_request == SIGTERM && idle == 0)
+		return;
+
+	if (exit_request)
+	{
+		reset_variables();
+		child_exit(0);
+	}
+}
