@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header$
+ * $Header: /cvsroot/pgpool/pgpool-II/pcp_child.c,v 1.13 2010/03/06 12:54:01 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -58,7 +58,7 @@
 #define MAX_USER_PASSWD_LEN  128
 
 extern void pcp_set_timeout(long sec);
-static int exit_request; /* non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived */
+volatile sig_atomic_t pcp_exit_request; /* non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived */
 
 static RETSIGTYPE die(int sig);
 static PCP_CONNECTION *pcp_do_accept(int unix_fd, int inet_fd);
@@ -813,7 +813,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 static RETSIGTYPE
 die(int sig)
 {
-	exit_request = 1;
+	pcp_exit_request = 1;
 
 	pool_debug("PCP child receives shutdown request signal %d", sig);
 
@@ -967,7 +967,7 @@ user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len)
 	char packet_password[MAX_USER_PASSWD_LEN+1];
 	char encrypt_buf[(MD5_PASSWD_LEN+1)*2];
 	char file_username[MAX_USER_PASSWD_LEN+1];
-	char file_password[MAX_USER_PASSWD_LEN+1];
+	char file_password[MD5_PASSWD_LEN+1];
 	char *index = NULL;
 	static char line[MAX_FILE_LINE_LEN+1];
 	int i, len;
